@@ -36,7 +36,13 @@
 #endif
 #include <netdb.h>
 #ifdef HAVE_PKG_hesiod
+#ifdef __cplusplus
+extern "C" {
+#endif
 #include <hesiod.h>
+#ifdef __cplusplus
+}
+#endif
 #endif
 
 #include <langinfo.h>
@@ -53,6 +59,8 @@
 #include "getaddrinfo.h"
 #include "tunable.h"
 
+#include "sdump.h"
+
 /* throw types for runtime errors */
 #define THROW_TIMEOUT	1		/* server timed out */
 
@@ -246,15 +254,17 @@ const char *canonical;  /* server name */
     if (retval) {
 #ifdef HEIMDAL
       if (err_ret && err_ret->e_text) {
+	  char *t = err_ret->e_text;
+	  char *tt = sdump(t, strlen(t));
           report(stderr, GT_("krb5_sendauth: %s [server says '%s']\n"),
-                 error_message(retval),
-                 err_ret->e_text);
+                 error_message(retval), tt);
+	  free(tt);
 #else
       if (err_ret && err_ret->text.length) {
-          report(stderr, GT_("krb5_sendauth: %s [server says '%*s']\n"),
-		 error_message(retval),
-		 err_ret->text.length,
-		 err_ret->text.data);
+	  char *tt = sdump(err_ret->text.data, err_ret->text.length);
+          report(stderr, GT_("krb5_sendauth: %s [server says '%s']\n"),
+		 error_message(retval), tt);
+	  free(tt);
 #endif
 	  krb5_free_error(context, err_ret);
       } else
@@ -501,14 +511,11 @@ static int fetch_messages(int mailserver_socket, struct query *ctl,
         }
 	if (msgcode == MSGLEN_OLD)
 	{
-  		/* To avoid flooding the syslog when using --keep,
-  		 * report "Skipped message" only when:
-  		 *  1) --verbose is on, or
-  		 *  2) fetchmail does not use syslog
-  		 */
-	    if (   (outlevel >= O_VERBOSE) ||
-	           (outlevel > O_SILENT && !run.use_syslog)
-	       )
+	    /*
+	     * To avoid flooding the logs when using --keep, report
+	     * skipping for old messages only when --flush is on.
+	     */
+	    if (outlevel > O_SILENT && ctl->flush)
 	    {
 		report_build(stdout, 
 			     GT_("skipping message %s@%s:%d"),
@@ -800,16 +807,11 @@ flagthemail:
 	}
 	else
 	{
-	    if (   (outlevel >= O_VERBOSE) ||
-         		/* To avoid flooding the syslog when using --keep,
-         		 * report "Skipped message" only when:
-         		 *  1) --verbose is on, or
-         		 *  2) fetchmail does not use syslog, or
-         		 *  3) the message was skipped for some other
-         		 *     reason than just being old.
-         		 */
-	           (outlevel > O_SILENT && (!run.use_syslog || msgcode != MSGLEN_OLD))
-	       )
+	    /*
+	     * To avoid flooding the logs when using --keep, report
+	     * skipping of new messages only.
+	     */
+	    if (outlevel > O_SILENT && msgcode != MSGLEN_OLD)
 	    report_complete(stdout, GT_(" not flushed\n"));
 
 	    /* maybe we mark this message as seen now? */
@@ -1073,25 +1075,6 @@ static int do_session(
 		    strlcpy(errbuf, strerror(err_no), sizeof(errbuf));
 		report_complete(stderr, ": %s\n", errbuf);
 
-#ifdef __UNUSED__
-		/* 
-		 * Don't use this.  It was an attempt to address Debian bug
-		 * #47143 (Notify user by mail when pop server nonexistent).
-		 * Trouble is, that doesn't work; you trip over the case 
-		 * where your SLIP or PPP link is down...
-		 */
-		/* warn the system administrator */
-		if (open_warning_by_mail(ctl) == 0)
-		{
-		    stuff_warning(iana_charset, ctl,
-			 GT_("Subject: Fetchmail unreachable-server warning."));
-		    stuff_warning(NULL, ctl, "");
-		    stuff_warning(NULL, ctl, GT_("Fetchmail could not reach the mail server %s:"),
-				  ctl->server.pollname);
-		    stuff_warning(NULL, ctl, errbuf, ctl->server.pollname);
-		    close_warning_by_mail(ctl, (struct msgblk *)NULL);
-		}
-#endif
 	    }
 	    err = PS_SOCKET;
 	    set_timeout(0);
@@ -1109,11 +1092,13 @@ static int do_session(
 	/* perform initial SSL handshake on open connection */
 	if (ctl->use_ssl &&
 		SSLOpen(mailserver_socket, ctl->sslcert, ctl->sslkey,
-		    ctl->sslproto, ctl->sslcertck, ctl->sslcertpath,
+		    ctl->sslproto, ctl->sslcertck,
+		    ctl->sslcertfile, ctl->sslcertpath,
 		    ctl->sslfingerprint, ctl->sslcommonname ?
 		    ctl->sslcommonname : realhost, ctl->server.pollname,
 		    &ctl->remotename) == -1)
 	{
+	    set_timeout(0);
 	    report(stderr, GT_("SSL connection failed.\n"));
 	    err = PS_SOCKET;
 	    goto cleanUp;
@@ -1162,7 +1147,9 @@ static int do_session(
 	stage = STAGE_GETAUTH;
 	if (ctl->server.base_protocol->getauth)
 	{
+	    set_timeout(mytimeout);
 	    err = (ctl->server.base_protocol->getauth)(mailserver_socket, ctl, buf);
+	    set_timeout(0);
 
 	    if (err != 0)
 	    {
@@ -1181,6 +1168,9 @@ static int do_session(
 			   ctl->server.truename,
 			   (ctl->wehaveauthed ? GT_(" (previously authorized)") : "")
 			);
+		    if (ctl->server.authenticate == A_ANY && !ctl->wehaveauthed) {
+			report(stderr, GT_("For help, see http://www.fetchmail.info/fetchmail-FAQ.html#R15\n"));
+		    }
 
 		    /*
 		     * If we're running in background, try to mail the
