@@ -5,15 +5,7 @@
 
 #include <sys/types.h>
 
-#ifdef HAVE_MD5_H
-#include <md5.h>
-#else
 #include "fetchmail.h"
-#ifndef HEADER_MD5_H
-/* Try to avoid clashes with OpenSSL */
-#define HEADER_MD5_H 
-#endif
-
 
 #if SIZEOF_INT == 4
 typedef unsigned int uint32;
@@ -30,12 +22,11 @@ struct MD5Context {
 void MD5Init(struct MD5Context *context);
 void MD5Update(struct MD5Context *context, const void *buf, unsigned len);
 void MD5Final(void *digest, struct MD5Context *context);
-void MD5Transform(uint32 buf[], uint32 const in[]);
+void MD5Transform(uint32 buf[4], uint32 const in[16]);
 
 /*
  * This is needed to make RSAREF happy on some MS-DOS compilers.
  */
 typedef struct MD5Context MD5_CTX;
 
-#endif
 #endif /* !MD5_H */
