@@ -241,15 +241,15 @@ int do_rfc1731(int sock, char *command, char *truename)
     while (authenticator.length & 7) {
 	authenticator.length++;
     }
-    des_pcbc_encrypt((des_cblock *)authenticator.dat,
-	    (des_cblock *)authenticator.dat, authenticator.length, schedule,
+    des_pcbc_encrypt((const unsigned char *)authenticator.dat,
+	    (unsigned char *)authenticator.dat, authenticator.length, schedule,
 	    &session, 1);
 
     to64frombits(buf1, authenticator.dat, authenticator.length);
 
     /* ship down the response, accept the server's error/ok indication */
     suppress_tags = TRUE;
-    result = gen_transact(sock, buf1, strlen(buf1));
+    result = gen_transact(sock, "%s", buf1);
     suppress_tags = FALSE;
     if (result)
 	return(result);
